//Script by Fierelier
//Donate: https://www.paypal.me/Fierelier/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using GTA;

namespace banScript
{
	public class banScript : Script
	{	
		public banScript()
		{
			this.Tick += new EventHandler(this.doBanTick);
			this.BindConsoleCommand("ban", new ConsoleCommandDelegate(banCommand), "<gamertag> - Adds a player to the ban-list");
			this.BindConsoleCommand("unban", new ConsoleCommandDelegate(unbanCommand), "<gamertag> - Removes a player from the ban-list");
			this.BindConsoleCommand("banlist", new ConsoleCommandDelegate(banlistCommand), "[gamertag] - Lists players that are banned");
			//Add kick command, add list command.
		}
		
		//Functions
		private static bool CompareLists<T>(List<T> aListA, List<T> aListB)
		{
			if (aListA == null || aListB == null || aListA.Count != aListB.Count)
				return false;
			if (aListA.Count == 0)
				return true;
			Dictionary<T, int> lookUp = new Dictionary<T, int>();
			// create index for the first list
			for(int i = 0; i < aListA.Count; i++)
			{
				int count = 0;
				if (!lookUp.TryGetValue(aListA[i], out count))
				{
					lookUp.Add(aListA[i], 1);
					continue;
				}
				lookUp[aListA[i]] = count + 1;
			}
			for (int i = 0; i < aListB.Count; i++)
			{
				int count = 0;
				if (!lookUp.TryGetValue(aListB[i], out count))
				{
					// early exit as the current value in B doesn't exist in the lookUp (and not in ListA)
					return false;
				}
				count--;
				if (count <= 0)
					lookUp.Remove(aListB[i]);
				else
					lookUp[aListB[i]] = count;
			}
			// if there are remaining elements in the lookUp, that means ListA contains elements that do not exist in ListB
			return lookUp.Count == 0;
		}
		
		private void banfileCheck()
		{
			if (File.Exists("banlist.txt")) { return; }
			try {
				File.Create("banlist.txt").Dispose();
			} catch { Game.DisplayText("Can not create banlist.txt"); }
		}
		
		//Script
		List<GTA.Player> oldPlayerList = new List<GTA.Player>();
		private void doBanTick(object sender, EventArgs e)
		{
			try{
			if (GTA.Multiplayer.Host.isHost == false) { return; }
			
			List<GTA.Player> playerList = new List<GTA.Player>();
			foreach (GTA.Player player in Game.PlayerList) {
				playerList.Add(player);
			}
			
			if (CompareLists(playerList,oldPlayerList)) { return; }
			
			oldPlayerList.Clear();
			foreach (GTA.Player player in playerList) {
				oldPlayerList.Add(player);
			}
			
			banfileCheck();
			List<string> banList = new List<string>();
			try {
				foreach (string line in File.ReadLines("banlist.txt")) {
					banList.Add(line.Replace("\n","").ToLower());
				}
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while checking ban-list:\n" +exc.ToString());
				return;
			}
			
			foreach (GTA.Player player in playerList) {
				if (player == Player) continue;
				foreach (string p in banList) {
					if (p == player.Name.ToLower()) {
						//try { GTA.Multiplayer.Host.KickPlayer(player); } catch {} -- broke af, don't use
						Game.Console.Print("[BanScript] Kicking " +p+ ".");
						try { GTA.Native.Function.Call("NETWORK_KICK_PLAYER",player.Index); } catch {}
						oldPlayerList.Clear();
					}
				}
			}
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Uncaught exception, please report this error:\n" +exc.ToString());
			}
			Wait(1000);
		}
		
		private void banCommand(ParameterCollection Parameter)
		{
			try{
			string banPlayer = Parameter.ToString(0).ToLower();
			
			if (banPlayer == "") {
				Game.Console.Print("[BanScript] Error: No player given.");
				return;
			}
			
			banfileCheck();
			List<string> banList = new List<string>();
			try {
				foreach (string line in File.ReadLines("banlist.txt")) {
					string newLine = line.Replace("\n","").ToLower();
					if (newLine == "") { continue; }
					banList.Add(newLine);
					if (newLine == banPlayer) {
						Game.Console.Print("[BanScript] Error! This player has already been added to the ban-list.");
						return;
					}
				}
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while checking ban-list:\n" +exc.ToString());
				return;
			}
			
			string newFileText = "";
			foreach (string player in banList) {
				newFileText = newFileText + player + "\n";
			}
			
			newFileText = newFileText + banPlayer;
			
			try {
				File.WriteAllText("banlist.txt",newFileText);
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while adding to ban-list:\n" +exc.ToString());
				return;
			}
			
			oldPlayerList.Clear();
			Game.Console.Print("[BanScript] Added " +banPlayer+ " to ban-list!");
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Uncaught exception, please report this error:\n" +exc.ToString());
			}
		}
		
		private void banlistCommand(ParameterCollection Parameter)
		{
			try{
			string searchPlayer = Parameter.ToString(0).ToLower();
			Game.Console.Print("[BanScript] Players on ban-list:");
			bool foundPlayers = false;
			banfileCheck();
			try {
				foreach (string line in File.ReadLines("banlist.txt")) {
					string newLine = line.Replace("\n","").ToLower();
					if (newLine == "") { continue; }
					if (searchPlayer == "") {
						Game.Console.Print(newLine);
						foundPlayers = true;
					} else {
						if (newLine.Contains(searchPlayer)) {
							Game.Console.Print(newLine);
							foundPlayers = true;
						}
					}
				}
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while checking ban-list:\n" +exc.ToString());
				return;
			}
			
			if (foundPlayers == false) {
				if (searchPlayer == "") {
					Game.Console.Print("[BanScript] No players on ban-list.");
				} else {
					Game.Console.Print("[BanScript] No players found containing '" +searchPlayer+ "'.");
				}
			}
			
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Uncaught exception, please report this error:\n" +exc.ToString());
			}
		}
		
		private void unbanCommand(ParameterCollection Parameter)
		{
			try{
			string banPlayer = Parameter.ToString(0).ToLower();
			
			if (banPlayer == "") {
				Game.Console.Print("[BanScript] Error: No player given.");
				return;
			}
			
			banfileCheck();
			
			bool foundPlayer = false;
			List<string> banList = new List<string>();
			try {
				foreach (string line in File.ReadLines("banlist.txt")) {
					string newLine = line.Replace("\n","").ToLower();
					if (newLine == "") { continue; }
					if (newLine == banPlayer) { foundPlayer = true; continue; }
					banList.Add(newLine);
				}
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while checking ban-list:\n" +exc.ToString());
				return;
			}
			
			if (foundPlayer == false) {
				Game.Console.Print("[BanScript] Error, could not find player!");
				return;
			}
			
			string newFileText = "";
			foreach (string player in banList) {
				newFileText = newFileText + player + "\n";
			}
			
			if (newFileText.Length-1 > -1) { newFileText = newFileText.Remove(newFileText.Length-1); }
			
			try {
				File.WriteAllText("banlist.txt",newFileText);
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Error occured while adding to ban-list:\n" +exc.ToString());
				return;
			}
			
			oldPlayerList.Clear();
			Game.Console.Print("[BanScript] Removed " +banPlayer+ " from ban-list!");
			} catch(Exception exc) {
				Game.Console.Print("[BanScript] Uncaught exception, please report this error:\n" +exc.ToString());
			}
		}
	}
}